﻿Type=Class
Version=5.5
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module

#Region Events
#Event: Account(meta As Map)
#Event: CurrentAccount(meta As Map)
#Event: SpaceUsage(meta As Map)
#Event: MetaData(meta As Map)
#Event: ListFolder(meta As Map)
#Event: LatestCursor(meta As Map)
#Event: FileDownloadFinished(session As Map)
#Event: FileUpload(session As Map)
#Event: UploadSessionFinish(session As Map)
#Event: UploadSessionStatus(session As Map)
#Event: FileSearch(result As Map)
#Event: CreateFolder(path_lower As String, name As String)
#Event: Copy(meta As Map)
#Event: Move(meta As Map)
#Event: Delete(meta As Map)
#Event: Thumbnail(bmp As Bitmap, Tag As Object)
#Event: Preview(meta As Map)
#Event: Revisions(revs As Map)
#Event: GetSharedLinks(meta As Map)
#Event: CreateSharedLink(meta As Map)
#Event: RevokeSharedLink(meta As Map)
#Event: DownloadLink(meta As Map)
#Event: ShareFile(meta As Map)
#Event: GetURLJob(meta As Map)
#Event: Delta(meta As Map)
#Event: SaveURL(meta As Map)
#Event: FolderMetadata(meta As Map)
#Event: ListFolders(folders As Map)
#Event: ShareFolder(folders As Map)
#Event: UpdateFolderPolicy(meta As Map)
#Event: UnshareFolder(meta As Map)
#Event: TransferFolder(meta As Map)
#Event: UnmountFolder(meta As Map)
#Event: MountFolder(meta As Map)
#Event: ShareJobStatus(meta As Map)
#Event: JobStatus(meta As Map)
#Event: AddFolderMember(meta As Map)
#End Region

Sub Class_Globals
	Private mAccessToken As String	
	Private mApiV1url As String = "https://api.dropbox.com/1/"
	Private mApiV1notifyurl As String = "https://api-notify.dropbox.com/1/"
	Private mApiV1urlcontent As String = "https://api-content.dropbox.com/1/"
	Private mApiV2url As String = "https://api.dropboxapi.com/2/"
	Private mApiV2urlContent As String = "https://content.dropboxapi.com/2/"
																				
	Private mCallback As Object
	Private mEvent As String
	Private mDebug As Boolean
	Private mChunkedUploadPos As Int
	Private mUploadlist As List
	Private mVersion As String = "0.90ß"
End Sub

' Initialize the Dropbox Class.
' Parameters
' CallbackModule
' EventName
' AccessToken
' debug 
'<code>Example:
'dbx.GetAccount("454365363464")</code>
Public Sub Initialize(CallbackModule As Object, EventName As String, AccessToken As String, debug As Boolean) 
	mCallback = CallbackModule
	mEvent = EventName
	mAccessToken = AccessToken
	mDebug = debug
	mUploadlist.Initialize
End Sub

Public Sub GetVersion As String
	Return mVersion
End Sub
public Sub logVersionInfo()
	LogColor("**********************",Colors.Blue)
	LogColor("DrobBox Class V2",Colors.Blue)
	LogColor("implementing DbxAPI",Colors.Blue)
	LogColor("-> API V2-beta2 <-",Colors.Blue)
	LogColor("Class-Version: "&mVersion,Colors.Blue)
	LogColor("**********************",Colors.Blue)
	LogColor("Author: DonManfred",Colors.Blue)
	LogColor("Realname: Manfred Ssykor",Colors.Blue)
	LogColor("eMail: msy1912@gmail.com",Colors.Blue)
	LogColor("**********************",Colors.Blue)
End Sub


'
' ========================
' API V2
' ========================
'
'
'users
'/get_account										OK
'/get_current_account						OK
'/get_space_usage								OK
'/get_account_batch							MISSING

'files
'/get_metadata									OK
'/list_folder/longpoll					MISSING
'/list_folder										OK
'/list_folder/Continue					OK
'/list_folder/get_latest_cursor	OK
'/Download											OK
'/upload_session/start					OK
'/upload_session/append					OK
'/upload_session/finish					OK
'/Upload												OK
'/Search												OK
'/create_folder									OK
'/Delete												OK
'/permanently_delete						MISSING
'/copy													OK
'/move													OK
'/get_thumbnail									OK
'/get_preview										NOT WORKING AS YET
'/list_revisions								OK
'/restore												MISSING
'

'sharing
'/get_shared_links							OK
'/create_shared_link						OK
'/revoke_shared_link						OK
'/get_folder_metadata						OK
'/list_folders									OK
'/share_folder									OK
'/update_folder_policy					OK
'/unshare_folder								OK
'/transfer_folder								UNTESTED
'/unmount_folder								UNTESTED
'/mount_folder									UNTESTED
'/add_folder_member							UNTESTED (probably not working as yet as of missing member details
'/check_share_job_status				OK
'/check_job_status							OK
'/remove_folder_member					MISSING
'/update_folder_member					MISSING
'/relinquish_folder_membership	MISSING

' MISSING AT ALL
' /restore
' /get_account_batch
' /ListFolderLongpoll
'
' NOT WORKING:
' /get_preview
'
' ========================
' New in V0.90ß (12/05/15)
' sharing:
' ->get_shared_links							
' ->create_shared_link						
' ->revoke_shared_link						
' ->get_folder_metadata						
' ->list_folders									
' ->share_folder		
' ->update_folder_policy							
' ->unshare_folder
' ->transfer_folder
' ->unmount_folder
' ->mount_folder
' ->check_share_job_status
'
' ========================
' New in V0.87ß (9/9/2015)
' - getsharedlinks(path as String)
' - CreateSharedLink(path As String, shorturl As Boolean)




'Get information about a user's account.
' The Event 
'<code>Example:
'dbx.GetAccount("454365363464")</code>
Sub GetAccount(accountID As String)
	Dim job As HttpJob
	job.Initialize("getaccount",Me)
	job.PostString(mApiV2url&"users/get_account",$"{"account_id":"${accountID}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetAccount(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	'Log(res)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If IsPaused(mCallback) Then
	Else
		If SubExists(mCallback, mEvent & "_Account") Then
			CallSub2(mCallback, mEvent & "_Account", root)
		End If
	End If
End Sub

'Get information about the current user's account.
'<code>Example:
'dbx.GetCurrentAccount()</code>
Sub GetCurrentAccount()
	Dim job As HttpJob
	job.Initialize("getcurrentaccount",Me)
	job.PostString(mApiV2url&"users/get_current_account",Null)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetCurrentAccount(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim accreferral_link As String = root.Get("referral_link")
	Dim accaccount_id As String = root.Get("account_id")
	Dim accemail As String = root.Get("email")
	Dim acclocale As String = root.Get("locale")
	Dim accname As Map = root.Get("name")
	Dim accdisplay_name As String = accname.Get("display_name")
	Dim accfamiliar_name As String = accname.Get("familiar_name")
	Dim accsurname As String = accname.Get("surname")
	Dim accgiven_name As String = accname.Get("given_name")
	Dim is_paired As String = root.Get("is_paired")
	Dim account_type As Map = root.Get("account_type")
		Dim tag As String = account_type.Get(".tag")
	Dim country As String = root.Get("country") 
	If SubExists(mCallback, mEvent & "_CurrentAccount") Then
		CallSub2(mCallback, mEvent & "_CurrentAccount", root)
	End If
End Sub


Sub GetSpaceUsage()
	Dim job As HttpJob
	job.Initialize("getspaceusage",Me)
	job.PostString(mApiV2url&"users/get_space_usage",Null)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetSpaceUsage(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_SpaceUsage") Then
		CallSub2(mCallback, mEvent & "_SpaceUsage", root)
	End If
End Sub

Sub GetMetadata(path As String)
	Dim job As HttpJob
	job.Initialize("getmetadata",Me)
	job.Tag = path
	job.PostString(mApiV2url&"files/get_metadata",$"{"path":"${path}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetMetadata(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim path_lower As String = root.Get("path_lower")
	Dim rev As String = root.Get("rev")
	Dim name As String = root.Get("name")
	Dim size As Int = root.Get("size")
	Dim tag As String = root.Get(".tag")
	Dim client_modified As String = root.Get("client_modified")
	Dim server_modified As String = root.Get("server_modified") 
	root.Put("Tag",Job.Tag)
	
	If SubExists(mCallback, mEvent & "_MetaData") Then
		CallSub2(mCallback, mEvent & "_MetaData", root)
	End If
End Sub

Sub Listfolder(path As String)
	Dim job As HttpJob
	job.Initialize("listfolder",Me)
	job.Tag = path
	job.PostString(mApiV2url&"files/list_folder",$"{"path":"${path}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleListfolder(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim cursor As String = root.Get("cursor")
	Dim has_more As String = root.Get("has_more")
	Dim entries As List = root.Get("entries")
	For Each colentries As Map In entries
		Dim path_lower As String = colentries.Get("path_lower")
		Dim name As String = colentries.Get("name")
		Dim tag As String = colentries.Get(".tag")
	Next 
	If SubExists(mCallback, mEvent & "_ListFolder") Then
		CallSub2(mCallback, mEvent & "_ListFolder", root)
	End If
End Sub

Sub ListfolderContinue(cursor As String)
	Dim job As HttpJob
	job.Initialize("listfoldercontinue",Me)
	job.Tag = cursor
	job.PostString(mApiV2url&"files/list_folder/continue",$"{"cursor":"${cursor}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleListfolderContinue(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim cursor As String = root.Get("cursor")
	Dim has_more As String = root.Get("has_more")
	Dim entries As List = root.Get("entries")
	For Each colentries As Map In entries
		Dim path_lower As String = colentries.Get("path_lower")
		Dim name As String = colentries.Get("name")
		Dim tag As String = colentries.Get(".tag")
	Next 
	If SubExists(mCallback, mEvent & "_ListFolder") Then
		CallSub2(mCallback, mEvent & "_ListFolder", root)
	End If
End Sub

Sub GetLatestCursor(path As String, recursive As Boolean)
	Log($"GetLatestCursor()"$)
	Dim Job As HttpJob
	Job.Initialize("getlatestcursor",Me)
	Job.Tag = "GetLatestCursor"
	Job.PostString(mApiV2url&"files/list_folder/get_latest_cursor", $"{"path": "${path}",
    "recursive": ${recursive}}"$)
	Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	Job.GetRequest.SetHeader("Content-Type", "application/json")
	Job.GetRequest.SetContentType("application/json")
	Job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetLatestcursor(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_LatestCursor") Then
		CallSub2(mCallback, mEvent & "_LatestCursor", root)
	End If
End Sub


Sub Download(path As String, destpath As String, destfile As String)
	Dim job As HttpJob
	job.Initialize("filedownload",Me)
	job.Tag = File.Combine(destpath,destfile)
	LogColor("Download: "&job.Tag,Colors.Blue)
	job.Download(mApiV2urlContent&"files/download")
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"path":"${path}"}"$)
	job.GetRequest.SetHeader("Accept","application/vnd.dropbox-cors-hack")
End Sub
private Sub handleFiledownload(Job As HttpJob, res As String)
	Dim dest As String = GetFilename(Job.Tag)
	Dim OutStream As OutputStream
	Dim pth As String = GetPath(Job.Tag)
	Dim fle As String = GetFilename(Job.Tag)
	'Log("DownloadReady: "&Job.Tag)
	'Log("DestPath: "&pth)
	'Log("DestFilename: "&fle)
	OutStream = File.OpenOutput(pth, fle, False) ' Job.Tag is read to set the Original Filename we specify earlier in the creation of the Job
	File. Copy2(Job.GetInputStream,OutStream) ' save the file
	OutStream.Close
	Dim session As Map 
	session.Initialize
	session.Put("download",File.Combine(pth,fle))
	session.Put("Path",pth)
	session.Put("Filename",fle)
	Log(fle&" are written to "&pth) ' Write the Originalname to the log to see what happens ;-)
	If SubExists(mCallback, mEvent & "_FileDownloadFinished") Then
		CallSub2(mCallback, mEvent & "_FileDownloadFinished", session)
	End If
End Sub

Sub UploadSessionStart(sessionMap As Map)
	Dim job As HttpJob
	job.Initialize("UploadSessionInit",Me)
	job.Tag = sessionMap
	job.PostString(mApiV2urlContent&"files/upload_session/start","")
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Dropbox-API-Arg", Null)
	'
	'job.GetRequest.SetContentEncoding("text/plain")
	job.GetRequest.SetContentType("text/plain; charset=dropbox-cors-hack")
End Sub
private Sub handleUploadSessionInit(Job As HttpJob, res As String)
	'Log("JobDone.JobName: " & Job.JobName)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim session_id As String = root.Get("session_id") 			

	Dim session As Map = Job.Tag
	session.Put("sessionID",session_id)
	Log("SESSION: "&session)
	' {srcpath=/storage/emulated/0, srcfile=test.xyz, dstpath=/Export, dstfile=newfilename.zzz, offset=0, packetsize=0, sessionID=AAAAAAAAtfnfd4UP7uheww}
	Dim Job As HttpJob
	Job.Initialize("UploadSessionPostbytes",Me)
	If File.Exists(session.Get("srcpath"),session.Get("srcfile")) Then
		Dim raf As RandomAccessFile
		raf.Initialize(session.Get("srcpath"),session.Get("srcfile"),True)

		Dim filesize As Int = File.Size(session.Get("srcpath"),session.Get("srcfile"))
		session.Put("filesize",filesize)
		If session.Get("packetsize") = 0 Then
			session.Put("packetsize",4096)
		End If
		mChunkedUploadPos = session.Get("offset")
		Dim buffer(session.Get("packetsize")) As Byte 
		Dim bytesread As Int = 0
		bytesread = raf.ReadBytes(buffer,0,session.Get("packetsize"),session.Get("offset"))
		Log("ByteRead="&bytesread)
		raf.Close
		session.Put("offset",session.Get("offset")+bytesread)
		Job.Tag = session

		If SubExists(mCallback, mEvent & "_UploadSessionStatus") Then
			CallSub2(mCallback, mEvent & "_UploadSessionStatus", session)
		End If

		' Data() as Byte
		Job.PostBytes(mApiV2urlContent&"files/upload_session/append", buffer)
		Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
		Job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"session_id": "${session_id}","offset": ${mChunkedUploadPos}}"$)
		Job.GetRequest.SetContentType("application/octet-stream")
	Else
		Log("404 filenotfound: "&File.Combine(session.Get("srcpath"),session.Get("srcfile")))
	End If
	'Job.GetRequest.SetHeader("Content-Type", "application/json")
	'Job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleUploadSessionFinish(Job As HttpJob, res As String)
  Log("JobDone.JobName: " & Job.JobName)
	Dim session As Map = Job.Tag
	Log("Session="&session)
	Log(Job.GetString)
	If SubExists(mCallback, mEvent & "_UploadSessionFinish") Then
		CallSub2(mCallback, mEvent & "_UploadSessionFinish", session)
	End If
End Sub
private Sub handleUploadSessionPostBytes(Job As HttpJob, res As String)
	Log("JobDone.JobName: " & Job.JobName)
	Dim session As Map = Job.Tag
	Log("Session="&session)
	Dim session_id As String = session.Get("sessionID")
	If SubExists(mCallback, mEvent & "_UploadSessionStatus") Then
		CallSub2(mCallback, mEvent & "_UploadSessionStatus", session)
	End If
	
	Dim raf As RandomAccessFile
	raf.Initialize(session.Get("srcpath"),session.Get("srcfile"),True)

	Dim offset As Int = session.Get("offset")
	Log("UploadOffset: "&offset)

	
	
	Dim buffer(session.Get("packetsize")) As Byte 
	Dim bytesread As Int = 0
	bytesread = raf.ReadBytes(buffer,0,session.Get("packetsize"),session.Get("offset"))
	raf.Close
	Dim newoffset As Int = offset+bytesread
	session.Put("offset",newoffset)

	Log("NEW offset = "&newoffset)
		
	If newoffset = session.Get("filesize") Then
		LogColor("Last chunk to upload... Should be Uploadsession_finish instead of append now",Colors.Blue)
		Log("UploadOffset = filesize!!! "&offset)		
		Dim Job As HttpJob
		Job.Initialize("UploadSessionFinish",Me)
		Job.PostBytes(mApiV2urlContent&"files/upload_session/finish", buffer)
		Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
		Job.Tag = session

		'curl -X POST https://content.dropboxapi.com/2-beta-2/files/upload_session/finish \
		'    --header "Authorization: Bearer <access-token>" \
		'    --header "Content-Type: application/octet-stream" \
		'    --header "Dropbox-API-Arg: {\"cursor\": {\"session_id\": \"1234faaf0678bcde\",\"offset\": 0},\"commit\": {\"path\": \"/Homework/math/Matrices.txt\",\"mode\": \"add\",\"autorename\": True,\"mute\": False}}" \

		Log($"{"cursor": {"session_id": "${session_id}","offset": ${offset}},"commit": {"path": "${session.Get("dstpath")}/${session.Get("dstfile")}","mode": "add","autorename": true,"mute": false}}"$)
		Job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"cursor": {"session_id": "${session_id}","offset": ${offset}},"commit": {"path": "${session.Get("dstpath")}/${session.Get("dstfile")}","mode": "add","autorename": true,"mute": false}}"$)
		
		Job.GetRequest.SetContentType("application/octet-stream")
	Else
		Dim Job As HttpJob
		Job.Initialize("UploadSessionPostbytes",Me)
		Job.Tag = session
		Job.PostBytes(mApiV2urlContent&"files/upload_session/append", buffer)
		Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
		Log($"{"session_id": "${session_id}","offset": ${offset}}"$)
		Job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"session_id": "${session_id}","offset": ${offset}}"$)
		Job.GetRequest.SetContentType("application/octet-stream")
	End If
	' Data() as Byte
	
	
End Sub

' work in progress
Sub Upload(localpath As String, localFilename As String, dstpath As String, dstfilename As String)
	If mUploadlist.Size > 0 Then
		'if mUploadlist.IndexOf(path) 
	End If
	Dim session As Map 
	session.Initialize
	session.Put("srcpath",localpath)
	session.Put("srcfile",localFilename)
	session.Put("dstpath",dstpath)
	session.Put("dstfile",dstfilename)
	session.Put("status","init")
	
	Dim job As HttpJob
	job.Initialize("fileupload",Me)
	job.Tag = session
	Log($"{"path":"${File.Combine(dstpath,dstfilename)}", "mode": "overwrite"}"$)
	job.PostFile(mApiV2urlContent&"files/upload",localpath,localFilename)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	'job.GetRequest.SetHeader("Content-Type", "application/octet-stream")
	job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"path":"${File.Combine(dstpath,dstfilename)}", "mode": "overwrite"}"$)
	job.GetRequest.SetContentType("application/octet-stream")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleFileUpload(Job As HttpJob, res As String)
	Dim session As Map 
	session.Initialize
	Log("RES:"&res)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	session = root
	Dim path As String = root.Get("path_lower")
	Dim rev As String = root.Get("rev")
	Dim name As String = root.Get("name")
	Dim size As Int = root.Get("size")
	Dim client_modified As String = root.Get("client_modified")
	Dim server_modified As String = root.Get("server_modified") 
	Log("dbxSession="&session)
	If SubExists(mCallback, mEvent & "_FileUpload") Then
		CallSub2(mCallback, mEvent & "_FileUpload", session)
	End If
End Sub

Sub Search(path As String, searchquery As String)
	Dim job As HttpJob
	job.Initialize("filesearch",Me)
	job.PostString(mApiV2url&"files/search",$"{"path":"${path}", "query": "${searchquery}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleSearch(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim matches As List = root.Get("matches")
	For Each colmatches As Map In matches
		Dim match_type As Map = colmatches.Get("match_type")
		Dim match_type_tag As String = match_type.Get(".tag")
		Dim metadata As Map = colmatches.Get("metadata")
		Dim path_lower As String = metadata.Get("path_lower")
		Dim name As String = metadata.Get("name")
		Dim metatag As String = metadata.Get(".tag")
	Next
	Dim more As String = root.Get("more")
	Dim start As Int = root.Get("start") 
	If SubExists(mCallback, mEvent & "_FileSearch") Then
		CallSub2(mCallback, mEvent & "_FileSearch", root)
	End If
End Sub

Sub CreateFolder(path As String)
	Dim job As HttpJob
	job.Initialize("create_folder",Me)
	job.PostString(mApiV2url&"files/create_folder",$"{"path":"${path}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleCreateFolder(Job As HttpJob, res As String)
	LogColor(res,Colors.Magenta)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim path_lower As String = root.Get("path_lower")
	Dim name As String = root.Get("name") 			
	If SubExists(mCallback, mEvent & "_CreateFolder") Then
		CallSub3(mCallback, mEvent & "_CreateFolder", path_lower, name)
	End If	
End Sub

Sub copy(fromPath As String, toPath As String)
	Dim job As HttpJob
	job.Initialize("copy",Me)
	job.PostString(mApiV2url&"files/copy",$"{"from_path":"${fromPath}","to_path":"${toPath}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleCopy(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_copy") Then
		CallSub2(mCallback, mEvent & "_copy", root)
	End If
End Sub

Sub move(fromPath As String, toPath As String)
	Dim job As HttpJob
	job.Initialize("copy",Me)
	job.PostString(mApiV2url&"files/move",$"{"from_path":"${fromPath}","to_path":"${toPath}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleMove(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_move") Then
		CallSub2(mCallback, mEvent & "_move", root)
	End If

End Sub



' Not yet finished....
Sub Delete(path As String)
	Dim job As HttpJob
	job.Initialize("delete",Me)
	job.PostString(mApiV2url&"files/delete",$"{"path":"${path}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleDelete(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_delete") Then
		CallSub2(mCallback, mEvent & "_delete", root)
	End If
	
End Sub


' Gets a thumbnail for an image.
'Parameters
' format - jpeg (default) Or png. 
' For images that are photos, jpeg should be preferred, 
' While png Is better For screenshots And digital art.
'
' size One of the following values (default: s):
'value	dimensions (px)
' xs		32x32
' s			64x64
' m			128x128
' l			640x480
' xl		1024x768
'
Sub GetThumbnail(path As String, format As String, size As String)
	Dim job As HttpJob
	job.Initialize("getthumbnail",Me)
	job.Tag = path
	job.PostString(mApiV2urlContent&"files/get_thumbnail","")
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"path":"${path}","format":"${format}","size":"${size}"}"$)
	job.GetRequest.SetContentEncoding("text/plain")
	job.GetRequest.SetContentType("")
End Sub
private Sub handleGetThumbnail(Job As HttpJob, res As String)
	'Dim bmp As Bitmap
	'bmp.Initialize3(Job.GetBitmap)
	If SubExists(mCallback, mEvent & "_Thumbnail") Then
		CallSub3(mCallback, mEvent & "_Thumbnail", Job.GetBitmap, Job.Tag)
	End If
End Sub



' Not working as yet!
' It returns a html page instead of a jsonstring
Sub GetPreview(path As String)
	Dim job As HttpJob
	job.Initialize("get_preview",Me)
	job.Tag = path
	job.PostString(mApiV2urlContent&"files/get_preview", "")
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"path": "${path}"}"$)
	job.GetRequest.SetContentEncoding("text/plain")
	'job.GetRequest.SetHeader("Accept","application/json")
	job.GetRequest.SetContentType("")
End Sub
private Sub handleGetPreview(Job As HttpJob, res As String)
	Log("get_preview: " & res)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_Preview") Then
		CallSub2(mCallback, mEvent & "_Preview", root)
	End If
End Sub


Sub GetRevisions(path As String, revlimit As Int)
	Dim job As HttpJob
	job.Initialize("getrevisions",Me)
	job.Tag = path
	Dim json As String = $"{"path":"${path}","limit":${revlimit}}"$
	Log(json)
	job.PostString(mApiV2url&"files/list_revisions",json)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetRevisions(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_Revisions") Then
		CallSub2(mCallback, mEvent & "_Revisions", root)
	End If
End Sub

Sub GetSharedLinks(path As String)
	Dim job As HttpJob
	job.Initialize("getsharedlinks",Me)
	job.Tag = path
	Dim json As String = $"{"path":"${path}"}"$
	Log(json)
	job.PostString(mApiV2url&"sharing/get_shared_links",json)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetSharedLinks(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_GetSharedLinks") Then
		CallSub2(mCallback, mEvent & "_GetSharedLinks", root)
	End If
End Sub

Sub CreateSharedLink(path As String, shorturl As Boolean)
	Dim job As HttpJob
	job.Initialize("CreateSharedLink",Me)
	job.Tag = path
	Dim json As String = $"{"path":"${path}","short_url":${shorturl}}"$
	Log(json)
	job.PostString(mApiV2url&"files/get_shared_links",json)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleCreateSharedLink(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_CreateSharedLink") Then
		CallSub2(mCallback, mEvent & "_CreateSharedLink", root)
	End If
End Sub

Sub RevokeSharedLink(url As String)
	Dim job As HttpJob
	job.Initialize("RevokeSharedLink",Me)
	job.Tag = url
	Dim json As String = $"{"url":"${url}"}"$
	Log(json)
	job.PostString(mApiV2url&"files/revoke_shared_link",json)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleRevokeSharedLink(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_RevokeSharedLink") Then
		CallSub2(mCallback, mEvent & "_RevokeSharedLink", root)
	End If
End Sub


Sub GetFolderMetadata(SharedFolderID As String, IncludeMembership As Boolean)
	Dim job As HttpJob
	job.Initialize("GetFolderMetadata",Me)
	job.Tag = SharedFolderID
	Dim json As String = $"{"shared_folder_id": "${SharedFolderID}","include_membership": ${IncludeMembership}"$
	Log(json)
	job.PostString(mApiV2url&"sharing/get_folder_metadata",json)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetFolderMetadata(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_FolderMetadata") Then
		CallSub2(mCallback, mEvent & "_FolderMetadata", root)
	End If
End Sub


Sub ListFolders(include_membership As Boolean)
	Dim job As HttpJob
	job.Initialize("listfolders",Me)
	job.Tag = "listfolders"
	job.PostString(mApiV2url&"sharing/list_folders",$"{"include_membership":${include_membership}}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleListfolders(Job As HttpJob, res As String)
	Log("handleListfolders")
	Log(Job.GetString)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_ListFolders") Then
		CallSub2(mCallback, mEvent & "_ListFolders", root)
	End If
End Sub

'Parameters:
' path String
' The path To the folder To share. If it does Not exist, Then a new one Is created.
'
' member_policy String 
' MemberPolicy Who can be a member of this shared folder. 
' Policy governing who can be a member of a shared folder.
' Only applicable To folders owned by a user on a team.
' The value will be one of the following datatypes. 
' New values may be introduced As our API evolves.
'
' "team" Only a teammate can become a member.
' "anyone" Anyone can become a member.
'
' acl_update_policy String
' AclUpdatePolicy Who can add And remove members of this shared folder.
' Policy governing who can change a shared folder's access control 
' list (ACL). In other words, who can add, remove, or change the 
' privileges of members.The value will be one of the following 
' datatypes. New values may be introduced as our API evolves.
'
' "owner" Only the owner can update the ACL.
' "editors" Any editor can update the ACL. 
'           This may be further restricted To editors on the same team.
'
' shared_link_policy String
' The policy To apply To shared links created For 
' content inside this shared folder.
' Policy governing who can view shared links.The value 
' will be one of the following datatypes. New values 
' may be introduced As our API evolves.
'
' "anyone" Links can be shared with anyone.
' "members" Links can only be shared among 
' members of the shared folder.
'
' force_async Boolean 
' Whether To force the share To happen asynchronously. 
' The default For this field Is False.
'	
Sub ShareFolder(Path As String, MemberPolicy As String, acl_update_policy As String, shared_link_policy As String, forceAsync As Boolean)
	Dim job As HttpJob
	job.Initialize("sharefolder",Me)
	job.Tag = "listfolders"
	'
	'  "{"path": "/test folder", "member_policy": {".tag": "team"}, "acl_update_policy": {".tag": "owner"}, "shared_link_policy": {".tag": "members"}, "force_async": false}"
	'
	job.PostString(mApiV2url&"sharing/share_folder",$"{"path": "${Path}", "member_policy": {".tag": "${MemberPolicy}"}, "acl_update_policy": {".tag": "${acl_update_policy}"}, "shared_link_policy": {".tag": "${shared_link_policy}"}, "force_async": ${forceAsync}}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleShareFolder(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_ShareFolder") Then
		CallSub2(mCallback, mEvent & "_ShareFolder", root)
	End If
End Sub


Sub update_folder_policy(SharedFolderID As String, MemberPolicy As String, acl_update_policy As String, shared_link_policy As String)
	Dim job As HttpJob
	job.Initialize("update_folder_policy",Me)
	job.Tag = "listfolders"
	'
	'  "{"path": "/test folder", "member_policy": {".tag": "team"}, "acl_update_policy": {".tag": "owner"}, "shared_link_policy": {".tag": "members"}, "force_async": false}"
	'
	job.PostString(mApiV2url&"sharing/update_folder_policy",$"{"shared_folder_id": "${SharedFolderID}", "member_policy": {".tag": "${MemberPolicy}"}, "acl_update_policy": {".tag": "${acl_update_policy}"}, "shared_link_policy": {".tag": "${shared_link_policy}"}}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleupdate_folder_policy(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_UpdateFolderPolicy") Then
		CallSub2(mCallback, mEvent & "_UpdateFolderPolicy", root)
	End If
End Sub



Sub UnshareFolder(SharedFolderID As String, leaveacopy As Boolean)
	Dim job As HttpJob
	job.Initialize("UnshareFolder",Me)
	job.Tag = "UnshareFolder"
	'
	'  "{"path": "/test folder", "member_policy": {".tag": "team"}, "acl_update_policy": {".tag": "owner"}, "shared_link_policy": {".tag": "members"}, "force_async": false}"
	'
	job.PostString(mApiV2url&"sharing/UnshareFolder",$"{"shared_folder_id": "${SharedFolderID}","leave_a_copy": ${leaveacopy}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleUnshareFolder(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_UnshareFolder") Then
		CallSub2(mCallback, mEvent & "_UnshareFolder", root)
	End If
End Sub


Sub transfer_folder(SharedFolderID As String, toDropboxID As String)
	Dim job As HttpJob
	job.Initialize("transfer_folder",Me)
	job.Tag = "transfer_folder"
	'
	'  "{"path": "/test folder", "member_policy": {".tag": "team"}, "acl_update_policy": {".tag": "owner"}, "shared_link_policy": {".tag": "members"}, "force_async": false}"
	'
	job.PostString(mApiV2url&"sharing/transfer_folder",$"{"shared_folder_id": "${SharedFolderID}","to_dropbox_id": ${toDropboxID}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleTransferFolder(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_TransferFolder") Then
		CallSub2(mCallback, mEvent & "_TransferFolder", root)
	End If
End Sub


Sub unmount_folder(SharedFolderID As String)
	Dim job As HttpJob
	job.Initialize("unmount_folder",Me)
	job.Tag = "unmount_folder"
	'
	'  "{"path": "/test folder", "member_policy": {".tag": "team"}, "acl_update_policy": {".tag": "owner"}, "shared_link_policy": {".tag": "members"}, "force_async": false}"
	'
	job.PostString(mApiV2url&"sharing/unmount_folder",$"{"shared_folder_id": "${SharedFolderID}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleUnmountFolder(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_UnmountFolder") Then
		CallSub2(mCallback, mEvent & "_UnmountFolder", root)
	End If
End Sub




Sub mount_folder(SharedFolderID As String)
	Dim job As HttpJob
	job.Initialize("mount_folder",Me)
	job.Tag = "mount_folder"
	'
	'  "{"path": "/test folder", "member_policy": {".tag": "team"}, "acl_update_policy": {".tag": "owner"}, "shared_link_policy": {".tag": "members"}, "force_async": false}"
	'
	job.PostString(mApiV2url&"sharing/mount_folder",$"{"shared_folder_id": "${SharedFolderID}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleMountFolder(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_MountFolder") Then
		CallSub2(mCallback, mEvent & "_MountFolder", root)
	End If
End Sub


Sub check_share_job_status(async_job_id As String)
	Dim job As HttpJob
	job.Initialize("check_share_job_status",Me)
	job.Tag = "check_share_job_status"
	'
	'  "{"path": "/test folder", "member_policy": {".tag": "team"}, "acl_update_policy": {".tag": "owner"}, "shared_link_policy": {".tag": "members"}, "force_async": false}"
	'
	job.PostString(mApiV2url&"sharing/check_share_job_status",$"{"async_job_id": "${async_job_id}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handlecheck_share_job_status(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_ShareJobStatus") Then
		CallSub2(mCallback, mEvent & "_ShareJobStatus", root)
	End If
End Sub


Sub check_job_status(async_job_id As String)
	Dim job As HttpJob
	job.Initialize("check_job_status",Me)
	job.Tag = "check_job_status"
	'
	'  "{"path": "/test folder", "member_policy": {".tag": "team"}, "acl_update_policy": {".tag": "owner"}, "shared_link_policy": {".tag": "members"}, "force_async": false}"
	'
	job.PostString(mApiV2url&"sharing/check_job_status",$"{"async_job_id": "${async_job_id}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handlecheck_job_status(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_JobStatus") Then
		CallSub2(mCallback, mEvent & "_JobStatus", root)
	End If
End Sub



Sub add_folder_member(shared_folder_id As String, email As String, dropbox_id As String, access_level As String, quiet As Boolean, custom_message As String)
	Dim job As HttpJob
	job.Initialize("add_folder_member",Me)
	job.Tag = "add_folder_member"
	'
	' curl -X POST https://api.dropboxapi.com/2/sharing/add_folder_member \
	'--header "Authorization: Bearer <get access token>" \
	'--header "Content-Type: application/json" \
	'--data "{\"shared_folder_id\": \"dbsfid:BCcDKIi3BO5uA9Kzv1v7I9MBJiqoXZXx7Fo\",\"members\": [{\"member\": {\".tag\": \"email\",\"email\": \"justin@example.com\"},\"access_level\": {\".tag\": \"editor\"}},{\"member\": {\".tag\": \"dropbox_id\",\"dropbox_id\": \"dbid:AAEufNrMPSPe0dMQijRP0N_aZtBJRm26W4Q\"},\"access_level\": {\".tag\": \"viewer\"}}],\"quiet\": False,\"custom_message\": \"Documentation For launch day\"}"
	'
	'
	'Hope this helps!
	'Regards,
	'Gregory
	'
	If email <> "" Then
		job.PostString(mApiV2url&"sharing/add_folder_member",$"{"shared_folder_id": "${shared_folder_id}","members": [{"member": {".tag": "email","email": "${email}"}, "access_level": {".tag": "${access_level}"}}],"quiet": ${quiet},"custom_message": "${custom_message}"}"$)
	Else	
		job.PostString(mApiV2url&"sharing/add_folder_member",$"{"shared_folder_id": "${shared_folder_id}","members": [{"member": {".tag": "dropbox_id","dropbox_id" : "${dropbox_id}"},"access_level": {".tag": "${access_level}"}}],"quiet": ${quiet},"custom_message": "${custom_message}"}"$)
	End If

	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleadd_folder_member(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_AddFolderMember") Then
		CallSub2(mCallback, mEvent & "_AddFolderMember", root)
	End If
End Sub






Sub JobDone(Job As HttpJob)
	If mDebug Then
		Log("JobDone("&Job.JobName&")")
	End If
  ProgressDialogHide

  If Job.Success Then
    Dim res As String
    res=Job.GetString
		If mDebug Then
  	  'Log("JobName: " & Job.JobName)
    	If (Job.JobName <> "filedownload") And (Job.JobName <> "getthumbnail") Then
				'Log("Server response: " & res)
			End If
		End If
    Select Job.JobName
			Case "copy"
				handleCopy(Job,res)
			Case "move"
				handleMove(Job,res)
			Case "delete"
				handleDelete(Job,res)
			Case "UploadSessionInit"
				handleUploadSessionInit(Job,res)
			Case "UploadSessionFinish"
				handleUploadSessionFinish(Job,res)
			Case "UploadSessionPostbytes"	
	  	  handleUploadSessionPostBytes(Job,res)
			Case "get_preview"
				handleGetPreview(Job,res)
			Case "getspaceusage"
				handleGetSpaceUsage(Job,res)
			Case "create_folder"
				handleCreateFolder(Job, res)
			Case "getthumbnail"
				handleGetThumbnail(Job,res)
			Case "getrevisions"
				handleGetRevisions(Job,res)
			Case "getlatestcursor"
				handleGetLatestcursor(Job,res)
			Case "getdelta"
        handleGetDelta(Job,res)
			Case "saveurljob"
				handleGetURLJob(Job,res)
			Case "saveurl"
				handleSaveURL(Job,res)
			Case "filedownload"
				handleFiledownload(Job,res)				
			Case "fileupload"
				handleFileUpload(Job,res)
			Case "downloadlink"
				handleGetDownloadLink(Job,res)
			Case "sharefile"
				handleGetShareFileLink(Job,res)
			Case "getmetadata"
				handleGetMetadata(Job, res)
			Case "filesearch"
				handleSearch(Job,res)
		  Case "listfolder"
				handleListfolder(Job, res)
	  	Case "listfoldercontinue"
				handleListfolderContinue(Job, res)
		  Case "getaccount"
				handleGetAccount(Job,res)
	  	Case "getcurrentaccount"
				handleGetCurrentAccount(Job,res)
			Case "getsharedlinks"
				handleGetSharedLinks(Job,res)
			Case "CreateSharedLink"
				handleCreateSharedLink(Job,res)
			Case "RevokeSharedLink"
				handleRevokeSharedLink(Job,res)
			Case "GetFolderMetadata"
				handleGetFolderMetadata(Job,res)
			Case "listfolders"
				handleListfolders(Job,res)
			Case "sharefolder"
				handleShareFolder(Job,res)
			Case "update_folder_policy"
				handleupdate_folder_policy(Job,res)
			Case "UnshareFolder"
				handleUnshareFolder(Job,res)
			Case "transfer_folder"
				handleTransferFolder(Job,res)
			Case "unmount_folder"
				handleUnmountFolder(Job,res)
			Case "mount_folder"
				handleMountFolder(Job,res)
			Case "check_share_job_status"
				handlecheck_share_job_status(Job,res)
			Case "check_job_status"
				handlecheck_job_status(Job,res)
	  End Select
  Else
   	Log("JobError: " & Job.ErrorMessage)
		ToastMessageShow("Error: " & Job.ErrorMessage,True)
  End If
	Job.Release           
End Sub
private Sub GetFilename(fullpath As String) As String
   Return fullpath.SubString(fullpath.LastIndexOf("/") + 1)
End Sub
private Sub GetBasename(fullpath As String) As String
   Dim filename As String
   filename = GetFilename(fullpath)
   Return filename.SubString2(0,filename.LastIndexOf("."))
End Sub
' Depends on: GetFilename
' Example
'Log(GetBasename("http://www.domain.tld/SomePage.html")) ' -> SomePage
private Sub GetPath(Path As String) As String
   Dim Path1 As String
   If Path = "/" Then
      Return "/"
   End If
   Dim L As Int = Path.LastIndexOf("/")
   If L = Path.Length - 1 Then
      'Strip the last slash
      Path1 = Path.SubString2(0,L)
   Else
      Path1 = Path
   End If
   L = Path.LastIndexOf("/")
   If L = 0 Then
      L = 1
   End If
   Return Path1.SubString2(0,L)
End Sub

' API V1 (Dropbox CORE)
private Sub request_token()
	'
	Dim job As HttpJob
	job.Initialize("gettoken",Me)
	
			 	
	job.PostString("https://api.dropbox.com/1/oauth/request_token",Null)
	job.Tag = "gettoken"
	'job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	'job.GetRequest.SetHeader("Content-Type", "application/json")
	'job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub authorize(responseType As String, clientid As String, rolerequired As String, force_reapprove As Boolean)
	'
	Dim job As HttpJob
	job.Initialize("oauth2",Me)
	
			 	
	job.Download("https://www.dropbox.com/1/oauth2/authorize?client_id="&clientid&"&force_reapprove="&force_reapprove&"&require_role="&rolerequired&"&response_type="&responseType)
	job.Tag = "oauth"
	'job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	'job.GetRequest.SetHeader("Content-Type", "application/json")
	'job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub

Sub GetDownloadLink(path As String)
	Dim job As HttpJob
	job.Initialize("downloadlink",Me)
	job.PostString(mApiV1url&"media/auto/"&path,Null)
	job.Tag = path
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetDownloadLink(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim expires As String = root.Get("expires")
	Dim url As String = root.Get("url") 
	root.Put("Tag",Job.Tag)
	If SubExists(mCallback, mEvent & "_DownloadLink") Then
		CallSub2(mCallback, mEvent & "_DownloadLink", root)
	End If
End Sub

Sub GetShareFileLink(path As String)
	Dim job As HttpJob
	job.Initialize("sharefile",Me)
	job.Tag = path
	job.PostString(mApiV1url&"shares/auto/"&path,Null)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetShareFileLink(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	Dim expires As String = root.Get("expires")
	Dim visibility As String = root.Get("visibility")
	Dim url As String = root.Get("url") 
	root.Put("Tag",Job.Tag)
	If SubExists(mCallback, mEvent & "_ShareFile") Then
		CallSub2(mCallback, mEvent & "_ShareFile", root)
	End If
End Sub


Sub GetURLJob(JobID As String)
	Log($"urljob="${JobID}""$)
	Dim Job As HttpJob
	Job.Initialize("saveurljob",Me)
	Job.Tag = JobID
	Job.PostString(mApiV1url&"save_url_job/"&JobID, "")
	Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	Job.GetRequest.SetHeader("Content-Type", "application/json")
	Job.GetRequest.SetContentType("application/json")
	Job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetURLJob(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_GetURLJob") Then
		CallSub2(mCallback, mEvent & "_GetURLJob", root)
	End If
End Sub


Sub GetDelta(cursor As String,locale As String, includeMediaInfo As Boolean)
	Log($"GetDelta()"$)
	Dim Job As HttpJob
	Job.Initialize("getdelta",Me)
	Job.Tag = "delta"
	'Log(mApiV1url&"delta?cursor="&cursor&"&locale="&locale&"&include_media_info="&includeMediaInfo)
	Job.PostString(mApiV1url&"delta?cursor="&cursor&"&locale="&locale&"&include_media_info="&includeMediaInfo, "")
	Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	Job.GetRequest.SetHeader("Content-Type", "application/json")
	Job.GetRequest.SetContentType("application/json")
	Job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleGetDelta(Job As HttpJob, res As String)
	'Dim OutStream As OutputStream
  'OutStream = File.OpenOutput(File.DirDefaultExternal, "getdelta.json", False)
  'File. Copy2(Job.GetInputStream,OutStream)
  'OutStream.Close
	'Log("getdelta.json written to "&File.DirDefaultExternal)			
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_Delta") Then
		CallSub2(mCallback, mEvent & "_Delta", root)
	End If
	
End Sub

Sub SaveURL(url As String, destpath As String)
	Log($"url="${url}""$)
	Dim job As HttpJob
	job.Initialize("saveurl",Me)
	job.Tag = url
	job.PostString(mApiV1url&"save_url/auto/"&destpath&"/?url="&url, $"{"url":"${url}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
private Sub handleSaveURL(Job As HttpJob, res As String)
	Dim parser As JSONParser
	parser.Initialize(res)
	Dim root As Map = parser.NextObject
	If SubExists(mCallback, mEvent & "_SaveURL") Then
		CallSub2(mCallback, mEvent & "_SaveURL", root)
	End If
	
End Sub
